from typing import Optional
import strawberry
import strawberry_django
from strawberry.django import auth
from gqlauth.utils import g_user
from django.contrib.auth import get_user_model

# project
from .types import UserType, UserFilter


@strawberry.django.type(model=get_user_model(), filters=UserFilter)
class UserQueries:
    user: Optional[UserType] = strawberry.django.field()
    users: list[UserType] = strawberry.django.field(filters=UserFilter)

    @strawberry.django.field
    def public_user(self, info) -> Optional[UserType]:
        user = g_user(info)
        if user.is_authenticated:
            return user

    @strawberry.django.field
    def me(self, info) -> Optional[UserType]:
        user = g_user(info)
        if not user.is_anonymous:
            return user



import typing

import strawberry
from gqlauth.settings import gqlauth_settings as app_settings
from gqlauth.bases.scalars import ExpectedErrorType

# ExpectedErrorType = app_settings.CUSTOM_ERROR_TYPE or ExpectedErrorType


@strawberry.interface
class OutputInterface:
    """
    A class to all public classes extend to
    padronize the output
    """

    success: bool
    errors: typing.Optional[ExpectedErrorType] = None



# Generated by Django 4.0.3 on 2022-04-04 07:15

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("gqlauth", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="userstatus",
            name="user",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="status",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from datetime import timedelta
from typing import Callable, TypeVar, Any
from dataclasses import dataclass, field
import strawberry
from django.conf import settings as django_settings
from faker import Faker

fake = Faker()


def default_text_factory():
    return " ".join([fake.city(), str(fake.pyint())])


_USER_NODE_FILTER_fieldS = {
    "email": ["exact"],
    "username": ["exact", "icontains", "istartswith"],
    "is_active": ["exact"],
    "status__archived": ["exact"],
    "status__verified": ["exact"],
    "status__secondary_email": ["exact"],
}

DjangoSetting = TypeVar('DjangoSetting')


@dataclass(slots=True)
class GqlAuthSettings:
    # if allow logging in without verification,
    # the register mutation will return a token
    ALLOW_LOGIN_NOT_VERIFIED: bool = False
    # mutation fields options
    LOGIN_OPTIONAL_FIELDS: list = field(default_factory=lambda: [])
    LOGIN_REQUIRE_CAPTCHA: bool = True
    LOGIN_REQUIRED_FIELDS: dict | list = field(
        default_factory=lambda: ["username", "password"]
    )
    # required fields on register, plus password1 and password2,
    # can be a dict like UPDATE_MUTATION_fieldS setting
    REGISTER_MUTATION_FIELDS: dict | list = field(
        default_factory=lambda: ["email", "username"]
    )
    REGISTER_MUTATION_FIELDS_OPTIONAL: list = field(default_factory=lambda: [])
    REGISTER_REQUIRE_CAPTCHA: bool = True
    # captcha stuff
    CAPTCHA_EXPIRATION_DELTA: timedelta = timedelta(seconds=120)
    CAPTCHA_MAX_RETRIES: int = 5
    ## any function that returns a string
    CAPTCHA_TEXT_FACTORY: Callable = default_text_factory
    ## a function that recives the original string vs user input
    CAPTCHA_TEXT_VALIDATOR: Callable[
        [str, str], bool
    ] = lambda original, received: bool(original == received)
    ## will show captcha every time you create one
    FORCE_SHOW_CAPTCHA: bool = False
    # optional fields on update account, can be list of fields
    UPDATE_MUTATION_FIELDS: dict | list = field(
        default_factory=lambda: {"first_name": str, "last_name": str}
    )
    # tokens
    EXPIRATION_ACTIVATION_TOKEN: timedelta = timedelta(days=7)
    EXPIRATION_PASSWORD_RESET_TOKEN: timedelta = timedelta(hours=1)
    EXPIRATION_SECONDARY_EMAIL_ACTIVATION_TOKEN: timedelta = timedelta(hours=1)
    EXPIRATION_PASSWORD_SET_TOKEN: timedelta = timedelta(hours=1)
    # email stuff
    EMAIL_FROM: DjangoSetting = \
        lambda: getattr(django_settings, "DEFAULT_FROM_EMAIL", "test@email.com")
    SEND_ACTIVATION_EMAIL: bool = True
    ## client: example.com/activate/token
    ACTIVATION_PATH_ON_EMAIL: str = "activate"
    ACTIVATION_SECONDARY_EMAIL_PATH_ON_EMAIL: str = "activate"
    ## client: example.com/password-set/token
    PASSWORD_SET_PATH_ON_EMAIL: str = "password-set"
    ## client: example.com/password-reset/token
    PASSWORD_RESET_PATH_ON_EMAIL: str = "password-reset"
    # email subjects templates
    EMAIL_SUBJECT_ACTIVATION: str = "email/activation_subject.txt"
    EMAIL_SUBJECT_ACTIVATION_RESEND: str = "email/activation_subject.txt"
    EMAIL_SUBJECT_SECONDARY_EMAIL_ACTIVATION: str = "email/activation_subject.txt"
    EMAIL_SUBJECT_PASSWORD_SET: str = "email/password_set_subject.txt"
    EMAIL_SUBJECT_PASSWORD_RESET: str = "email/password_reset_subject.txt"
    # email templates
    EMAIL_TEMPLATE_ACTIVATION: str = "email/activation_email.html"
    EMAIL_TEMPLATE_ACTIVATION_RESEND: str = "email/activation_email.html"
    EMAIL_TEMPLATE_SECONDARY_EMAIL_ACTIVATION: str = "email/activation_email.html"
    EMAIL_TEMPLATE_PASSWORD_SET: str = "email/password_set_email.html"
    EMAIL_TEMPLATE_PASSWORD_RESET: str = "email/password_reset_email.html"
    EMAIL_TEMPLATE_VARIABLES: dict = field(default_factory=lambda: {})
    # query stuff
    USER_NODE_EXCLUDE_FIELDS: dict | list = field(
        default_factory=lambda: ["password", "is_superuser"]
    )
    USER_NODE_FILTER_FIELDS: dict = field(
        default_factory=lambda: _USER_NODE_FILTER_fieldS
    )
    # others
    ## turn is_active to False instead
    ALLOW_DELETE_ACCOUNT: bool = False
    ## string path for email function wrapper, see the testproject example
    EMAIL_ASYNC_TASK: bool = False
    ## mutation error type
    CUSTOM_ERROR_TYPE: strawberry.scalar = "gqlauth.bases.scalars.ExpectedErrorType"
    ## registration with no password
    ALLOW_PASSWORDLESS_REGISTRATION: bool = False
    SEND_PASSWORD_SET_EMAIL: bool = False



from django.conf import settings as django_settings
from gqlauth.settings_type import GqlAuthSettings, DjangoSetting
from django.test.signals import setting_changed

import logging

logger = logging.getLogger(__name__)

gqlauth_settings: GqlAuthSettings = None

if user_settings := getattr(django_settings, "GQL_AUTH", False):
    if isinstance(user_settings, GqlAuthSettings):
        gqlauth_settings = user_settings

    else:
        raise Exception(
            f"GQL_AUTH settings should be of type "
            f"{GqlAuthSettings}, but you provided {type(user_settings)}"
        )

else:
    logger.warning(
        "GQL AUTH: You have not provided"
        "any custom gql auth settings falling back to defaults"
    )
    gqlauth_settings = GqlAuthSettings()

defaults = GqlAuthSettings()
# retain django_settings
for name, setting in gqlauth_settings.__dataclass_fields__.items():
    value = getattr(gqlauth_settings, name)
    if setting.type is DjangoSetting:
        if value is getattr(defaults, name):
            setattr(gqlauth_settings, name, value())
del defaults


# should be decorated probably
def reload_gqlauth_settings(*args, **kwargs):
    global gqlauth_settings
    setting, value = kwargs["setting"], kwargs["value"]
    if setting == "GQL_AUTH":
        gqlauth_settings = GqlAuthSettings(**value)


setting_changed.connect(reload_gqlauth_settings)



# Generated by Django 4.0.3 on 2022-04-04 07:15

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Captcha",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4, editable=False)),
                ("text", models.CharField(editable=False, max_length=50)),
                ("insert_time", models.DateTimeField(auto_now_add=True)),
                ("tries", models.IntegerField(default=0)),
            ],
        ),
        migrations.CreateModel(
            name="UserStatus",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("verified", models.BooleanField(default=False)),
                ("archived", models.BooleanField(default=False)),
                (
                    "secondary_email",
                    models.EmailField(blank=True, max_length=254, null=True),
                ),
            ],
        ),
    ]
